from datetime import datetime

from cpl_core.database import TableABC
from cpl_core.dependency_injection import ServiceProviderABC
from cpl_discord.service import DiscordBotServiceABC

from bot_data.model.auto_role import AutoRole


class AutoRoleRule(TableABC):
    def __init__(
        self,
        auto_role: AutoRole,
        discord_emoji_name: str,
        discord_role_id: int,
        created_at: datetime = None,
        modified_at: datetime = None,
        id=0,
    ):
        self._auto_role_rule_id = id
        self._auto_role = auto_role
        self._discord_emoji_name = discord_emoji_name
        self._discord_role_id = discord_role_id

        TableABC.__init__(self)
        self._created_at = created_at if created_at is not None else self._created_at
        self._modified_at = modified_at if modified_at is not None else self._modified_at

    @property
    def id(self) -> int:
        return self._auto_role_rule_id

    @property
    def auto_role(self) -> AutoRole:
        return self._auto_role

    @property
    def emoji_name(self) -> str:
        return self._discord_emoji_name

    @emoji_name.setter
    def emoji_name(self, value: str):
        self._discord_emoji_name = value

    @property
    def role_id(self) -> int:
        return self._discord_role_id

    @property
    @ServiceProviderABC.inject
    def role_name(self, bot: DiscordBotServiceABC) -> str:
        guild = bot.get_guild(self.auto_role.server.discord_id)
        return guild.get_role(self.role_id).name

    @role_id.setter
    def role_id(self, value: int):
        self._discord_role_id = value

    @staticmethod
    def get_select_all_string() -> str:
        return str(
            f"""
            SELECT * FROM `AutoRoleRules`;
        """
        )

    @staticmethod
    def get_select_by_id_string(id: int) -> str:
        return str(
            f"""
            SELECT * FROM `AutoRoleRules`
            WHERE `AutoRoleRuleId` = {id};
        """
        )

    @staticmethod
    def get_select_by_auto_role_id_string(id: int) -> str:
        return str(
            f"""
            SELECT * FROM `AutoRoleRules`
            WHERE `AutoRoleId` = {id};
        """
        )

    @property
    def insert_string(self) -> str:
        return str(
            f"""
            INSERT INTO `AutoRoleRules` (
                `AutoRoleId`, `DiscordEmojiName`, `DiscordRoleId`, `CreatedAt`, `LastModifiedAt`
            ) VALUES (
                {self._auto_role},
                '{self._discord_emoji_name}',
                {self._discord_role_id},
                '{self._created_at}',
                '{self._modified_at}'
            );
        """
        )

    @property
    def udpate_string(self) -> str:
        return str(
            f"""
            UPDATE `AutoRoleRules`
            SET `AutoRoleId` = {self._auto_role},
            `DiscordEmojiName` = {self._discord_emoji_name},
            `DiscordRoleId` = {self._discord_role_id},
            `LastModifiedAt` = '{self._modified_at}'
            WHERE `AutoRoleRuleId` = {self._auto_role_rule_id};
        """
        )

    @property
    def delete_string(self) -> str:
        return str(
            f"""
            DELETE FROM `AutoRoleRules`
            WHERE `AutoRoleRuleId` = {self._auto_role_rule_id};
        """
        )
