from bot_data.abc.auto_role_repository_abc import AutoRoleRepositoryABC
from bot_data.abc.client_repository_abc import ClientRepositoryABC
from bot_data.abc.known_user_repository_abc import KnownUserRepositoryABC
from bot_data.abc.level_repository_abc import LevelRepositoryABC
from bot_data.abc.server_repository_abc import ServerRepositoryABC
from bot_data.abc.user_joined_game_server_repository_abc import UserJoinedGameServerRepositoryABC
from bot_data.abc.user_joined_server_repository_abc import UserJoinedServerRepositoryABC
from bot_data.abc.user_joined_voice_channel_repository_abc import UserJoinedVoiceChannelRepositoryABC
from bot_data.abc.user_repository_abc import UserRepositoryABC
from bot_graphql.abc.query_abc import QueryABC
from bot_graphql.filter.auto_role_filter import AutoRoleFilter
from bot_graphql.filter.auto_role_rule_filter import AutoRoleRuleFilter
from bot_graphql.filter.client_filter import ClientFilter
from bot_graphql.filter.level_filter import LevelFilter
from bot_graphql.filter.server_filter import ServerFilter
from bot_graphql.filter.user_filter import UserFilter
from bot_graphql.filter.user_joined_game_server_filter import UserJoinedGameServerFilter
from bot_graphql.filter.user_joined_server_filter import UserJoinedServerFilter
from bot_graphql.filter.user_joined_voice_channel_filter import UserJoinedVoiceChannelFilter


class Query(QueryABC):
    def __init__(
        self,
        auto_roles: AutoRoleRepositoryABC,
        clients: ClientRepositoryABC,
        known_users: KnownUserRepositoryABC,
        levels: LevelRepositoryABC,
        servers: ServerRepositoryABC,
        user_joined_servers: UserJoinedServerRepositoryABC,
        user_joined_voice_channel: UserJoinedVoiceChannelRepositoryABC,
        user_joined_game_server: UserJoinedGameServerRepositoryABC,
        users: UserRepositoryABC,
    ):
        QueryABC.__init__(self, "Query")
        self._auto_roles = auto_roles
        self._clients = clients
        self._known_users = known_users
        self._levels = levels
        self._servers = servers
        self._user_joined_servers = user_joined_servers
        self._user_joined_voice_channels = user_joined_voice_channel
        self._user_joined_game_server = user_joined_game_server
        self._users = users

        self.add_collection("autoRole", lambda *_: self._auto_roles.get_auto_roles(), AutoRoleFilter)
        self.add_collection("autoRoleRule", lambda *_: self._auto_roles.get_auto_role_rules(), AutoRoleRuleFilter)
        self.add_collection("client", lambda *_: self._clients.get_clients(), ClientFilter)
        self.add_collection("knownUser", lambda *_: self._known_users.get_users())
        self.add_collection("level", lambda *_: self._levels.get_levels(), LevelFilter)
        self.add_collection("server", lambda *_: self._servers.get_servers(), ServerFilter)
        self.add_collection(
            "userJoinedServer", lambda *_: self._user_joined_servers.get_user_joined_servers(), UserJoinedServerFilter
        )
        self.add_collection(
            "userJoinedVoiceChannel",
            lambda *_: self._user_joined_voice_channels.get_user_joined_voice_channels(),
            UserJoinedVoiceChannelFilter,
        )
        self.add_collection(
            "userJoinedGameServer",
            lambda *_: self._user_joined_game_server.get_user_joined_game_servers(),
            UserJoinedGameServerFilter,
        )
        self.add_collection("user", lambda *_: self._users.get_users(), UserFilter)
