import discord
from cpl_core.database.context import DatabaseContextABC
from cpl_discord.command import DiscordCommandABC
from cpl_discord.service import DiscordBotServiceABC
from cpl_translation import TranslatePipe
from discord.ext import commands
from discord.ext.commands import Context

from bot_core.abc.client_utils_abc import ClientUtilsABC
from bot_core.abc.message_service_abc import MessageServiceABC
from bot_core.helper.command_checks import CommandChecks
from bot_core.logging.command_logger import CommandLogger
from bot_data.abc.server_repository_abc import ServerRepositoryABC
from bot_data.abc.user_repository_abc import UserRepositoryABC


class UnregisterGroup(DiscordCommandABC):
    def __init__(
        self,
        logger: CommandLogger,
        message_service: MessageServiceABC,
        bot: DiscordBotServiceABC,
        client_utils: ClientUtilsABC,
        servers: ServerRepositoryABC,
        users: UserRepositoryABC,
        db: DatabaseContextABC,
        t: TranslatePipe,
    ):
        DiscordCommandABC.__init__(self)

        self._logger = logger
        self._message_service = message_service
        self._bot = bot
        self._client_utils = client_utils
        self._servers = servers
        self._users = users
        self._db = db
        self._t = t

        self._logger.trace(__name__, f"Loaded command service: {type(self).__name__}")

    @commands.hybrid_group()
    @commands.guild_only()
    async def unregister(self, ctx: Context):
        pass

    @unregister.command()
    @commands.guild_only()
    @CommandChecks.check_is_ready()
    @CommandChecks.check_is_member_moderator()
    async def minecraft(self, ctx: Context, member: discord.Member):
        self._logger.debug(__name__, f"Received command register minecraft {ctx}")

        server = self._servers.get_server_by_discord_id(ctx.guild.id)
        user = self._users.get_user_by_discord_id_and_server_id(member.id, server.id)
        user.minecraft_id = None
        self._users.update_user(user)
        self._db.save_changes()

        await self._message_service.send_interaction_msg(
            ctx.interaction, self._t.transform("modules.base.unregister.success")
        )

        self._logger.trace(__name__, f"Finished register minecraft command")
