import { Component, OnInit } from "@angular/core";
import { catchError } from "rxjs/operators";
import { SoftwareVersion } from "src/app/models/config/software-version";
import { GuiService } from "src/app/services/gui/gui.service";
import { SettingsService } from "src/app/services/settings/settings.service";
import { SpinnerService } from "src/app/services/spinner/spinner.service";

@Component({
  selector: 'app-footer',
  templateUrl: './footer.component.html',
  styleUrls: ['./footer.component.scss']
})
export class FooterComponent implements OnInit {


  frontendVersion!: SoftwareVersion;
  backendVersion!: SoftwareVersion;

  constructor(
    private settings: SettingsService,
    private guiService: GuiService,
    private spinnerService: SpinnerService
  ) { }

  ngOnInit(): void {
    this.frontendVersion = this.settings.getWebVersion() ?? new SoftwareVersion('0', '0', '0');

    this.spinnerService.showSpinner();
    this.guiService.getApiVersion()
      .pipe(catchError(err => {
        this.spinnerService.hideSpinner();
        throw err;
      }))
      .subscribe(version => {
        this.spinnerService.hideSpinner();
        console.log('V', version)
        this.backendVersion = new SoftwareVersion(
          version.major,
          version.minor,
          version.micro
        );
      });
  }

}
